﻿Imports System
Imports System.IO
Imports System.Text

Public Class frmMain

    Public PvPrnWin32API As New clsPvPrnWin32API

    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        InitControls()

    End Sub

    Private Sub frmMain_Disposed(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Disposed
        PvPrnWin32API = Nothing
    End Sub


    Private Sub InitControls()

        Try
            ' Interface
            rdoIF_USB.Checked = True
            EnableIFControls(IUsb)

            ' Communication setting
            cmbLPT_Port.SelectedIndex = 0

            cmbCOM_Port.SelectedIndex = 0
            cmbCOM_Baudrate.SelectedItem = "115200"
            cmbCOM_Databits.SelectedItem = "8"
            cmbCOM_Parity.SelectedItem = "NONE"
            cmbCOM_Stopbits.SelectedItem = "1"

            ' Paper size 
            ' Default 4x6 inch -> (4*25.4) x (6*25.4) mm
            Dim value As Double = 4 * 25.4
            txtP_Width.Text = value.ToString("###0.###")
            value = 6 * 25.4
            txtP_Height.Text = value.ToString("###0.###")

            txtMargin_X.Text = "0"
            txtMargin_Y.Text = "0"

            cmbSpeed.SelectedIndex = 0
            cmbDensity.SelectedIndex = (14 - 1)

        Catch ex As Exception
            MsgBox(ex.ToString)
        End Try

    End Sub

    Private Sub EnableIFControls(ByVal nInterface As Integer)
        cmbLPT_Port.Enabled = (nInterface = IParallel)
        cmbCOM_Port.Enabled = (nInterface = ISerial Or nInterface = IBluetooth)
        cmbCOM_Baudrate.Enabled = (nInterface = ISerial)
        cmbCOM_Databits.Enabled = (nInterface = ISerial)
        cmbCOM_Parity.Enabled = (nInterface = ISerial)
        cmbCOM_Stopbits.Enabled = (nInterface = ISerial)
        txtNet_IPAddr.Enabled = (nInterface = ILan)
        txtNet_PortNum.Enabled = (nInterface = ILan)
    End Sub

    Private Sub btnPrinterStatus_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrinterStatus.Click
        If (ConnectPrinter() = False) Then
            Exit Sub
        End If

        Dim nStatus As Integer = PvPrnWin32API.checkStatus() ' ^cu command
        MessageBox.Show(GetStatusMsg(nStatus))

        PvPrnWin32API.DisconnectPrinter()
    End Sub

    Private Sub btnPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrint.Click
        If (ConnectPrinter() = False) Then
            Exit Sub
        End If

        PrintStart()
        PvPrnWin32API.DisconnectPrinter()

    End Sub

    Private Function GetStatusMsg(ByVal nStatus As Integer) As String
        Dim errMsg As String = ""
        Select Case nStatus
            Case ERR_CODE_NO_ERROR
                errMsg = "No Error"
            Case ERR_CODE_PAPER_EMPTY
                errMsg = "Paper Empty"
            Case ERR_CODE_COVER_OPEN
                errMsg = "Cover Open"
            Case ERR_CODE_MOTOR_OVER_HEAT
                errMsg = "Motor overheat"
            Case ERR_CODE_TPH_OVER_HEAT
                errMsg = "TPH overheat"
            Case ERR_CODE_AUTO_SENSING
                errMsg = "Auto-sensing failure"
            Case ERR_CODE_BOARD_OVER_HEAT
                errMsg = "Board overheat"
            Case ERR_CODE_WAIT_FOR_LABEL_TAKEN
                errMsg = "Wait for paper to be taken"
            Case ERR_CODE_PAUSED_LABEL
                errMsg = "Issued label is paused in peeler unit"
            Case ERR_CODE_PRINTING_LABEL
                errMsg = "Printing label in image buffer"
            Case ERR_CODE_BUILDING_LABEL
                errMsg = "Building label to be printed in the buffer"
            Case ERR_CODE_UNABLE_CONNECT
                errMsg = "Unable to connect the printer"
            Case Else
                errMsg = "Unknown error"
        End Select

        Return errMsg

    End Function

    Private Function ConnectPrinter() As Boolean
        Dim strPort As String = ""
        Dim nInterface As Integer = ISerial
        Dim nBaudrate As Integer = 115200
        Dim nDatabits As Integer = 8
        Dim nParity As Integer = 0
        Dim nStopbits As Integer = 0
        Dim nStatus As Integer = ERR_CODE_NO_ERROR

        If (rdoIF_Serial.Checked) Then
            nInterface = ISerial
            strPort = cmbCOM_Port.Text
            nBaudrate = Convert.ToInt32(cmbCOM_Baudrate.Text)
            nDatabits = Convert.ToInt32(cmbCOM_Databits.Text)
            nParity = cmbCOM_Parity.SelectedIndex
            nStopbits = cmbCOM_Stopbits.SelectedIndex
        ElseIf (rdoIF_Bluetooth.Checked) Then
            nInterface = IBluetooth
            strPort = cmbCOM_Port.Text
        ElseIf (rdoIF_Parallel.Checked) Then
            nInterface = IParallel
            strPort = cmbLPT_Port.Text
        ElseIf (rdoIF_USB.Checked) Then
            nInterface = IUsb
        ElseIf (rdoIF_Net.Checked) Then
            nInterface = ILan
            strPort = txtNet_IPAddr.Text
            nBaudrate = Convert.ToInt32(txtNet_PortNum.Text)
        End If

        nStatus = PvPrnWin32API.ConnectPrinter(nInterface, strPort, nBaudrate, nDatabits, nParity, nStopbits, 0)

        If (nStatus <> ERR_CODE_NO_ERROR) Then
            PvPrnWin32API.DisconnectPrinter()
            MessageBox.Show(GetStatusMsg(nStatus))
            Return False
        End If

        Return True
    End Function

    Private Function SendPrinterSettingCommand() As Boolean

        Dim bRetVal As Boolean = False

        ' 203 DPI : 1mm = 7.99 dots
        ' 300 DPI : 1mm = 11.81 dots
        ' 600 DPI : 1mm = 23.62 dots
        Dim resolution As Integer = PvPrnWin32API.GetPrinterDPI()
        Dim dotsPer1mm As Integer = Convert.ToInt32(Math.Round(resolution / 25.4F))
        Dim multiplier As Integer = 1

        If (resolution >= 600) Then
            multiplier = 3
        End If

        Dim nPaper_Width, nPaper_Height, nMarginX, nMarginY As Integer

        Try
            nPaper_Width = Convert.ToInt32(Double.Parse(txtP_Width.Text) * dotsPer1mm)
            nPaper_Height = Convert.ToInt32(Double.Parse(txtP_Height.Text) * dotsPer1mm)
            nMarginX = Convert.ToInt32(Double.Parse(txtMargin_X.Text) * dotsPer1mm)
            nMarginY = Convert.ToInt32(Double.Parse(txtMargin_Y.Text) * dotsPer1mm)
        Catch ex As Exception
            MessageBox.Show(ex.ToString())
            Return False
        End Try

        Dim nSpeed As Integer = 0
        If (cmbSpeed.SelectedIndex = 0) Then
            nSpeed = -1
        Else
            nSpeed = Convert.ToInt32(cmbSpeed.Text) - 1
        End If

        Dim nDensity As Integer = Convert.ToInt32(cmbDensity.Text)

        Dim nSensorType As Integer = SLCS_MEDIA_TYPE.GAP
        If (rdoBmark.Checked) Then
            nSensorType = SLCS_MEDIA_TYPE.BLACKMARK
        ElseIf (rdoContinuous.Checked) Then
            nSensorType = SLCS_MEDIA_TYPE.CONTINUOUS
        End If

        Dim nOrientation As Integer = IIf(rdoTop2Bottom.Checked, SLCS_ORIENTATION.TOP2BOTTOM, SLCS_ORIENTATION.BOTTOM2TOP)

        ' Clear Buffer of Printer
        bRetVal = PvPrnWin32API.ClearBuffer()

        ' Rewinder is not supported for PV3 and PV4
        'If (rdoRewind.Checked) Then
        'bRetVal = PvPrnWin32API.PrintDirect("RWDy", True)
        'End If

        ' Select international character set and code table.To
        bRetVal = PvPrnWin32API.SetCharacterset(SLCS_INTERNATIONAL_CHARSET.ICS_USA, SLCS_CODEPAGE.FCP_CP1252)

        ' Set Label and Printer
        bRetVal = PvPrnWin32API.SetConfigOfPrinter(nSpeed, nDensity, nOrientation, rdoCut.Checked, 1, True)

        '1 Inch : 25.4mm
        '1 mm   :  7.99 Dots (PV3, PV4) 
        bRetVal = PvPrnWin32API.SetPaper(nMarginX, nMarginY, nPaper_Width, nPaper_Height, nSensorType, 0, 16)

        If (rdoDt.Checked) Then ' Direct thermal
            bRetVal = PvPrnWin32API.PrintDirect("STd", True)
        Else ' Thermal transfer
            bRetVal = PvPrnWin32API.PrintDirect("STt", True)
        End If

        Return True
    End Function


    Private Function PrintStart() As Boolean
        Dim bRetVal As Boolean = False

        ' 203 DPI : 1mm = 7.99 dots
        ' 300 DPI : 1mm = 11.81 dots
        ' 600 DPI : 1mm = 23.62 dots
        Dim resolution As Integer = PvPrnWin32API.GetPrinterDPI()
        Dim dotsPer1mm As Integer = Convert.ToInt32(Math.Round(resolution / 25.4F))
        Dim multiplier As Integer = 1

        If (resolution >= 600) Then
            multiplier = 3
        End If

        SendPrinterSettingCommand()

        ' Prints string using TrueFont
        '  P1 : Horizontal position (X) [dot]
        '  P2 : Vertical position (Y) [dot]
        '  P3 : Font Name
        '  P4 : Font Size
        '  P5 : Rotation : (0 : 0 degree , 1 : 90 degree, 2 : 180 degree, 3 : 270 degree)
        '  P6 : Italic
        '  P7 : Bold
        '  P8 : Underline
        '  P9 : RLE (Run-length encoding)
        'PvPrnWin32API.PrintTrueFontLib(2 * dotsPer1mm, 5 * dotsPer1mm, "Arial", 14, 0, True, True, False, "Sample Label-1", False)
        PvPrnWin32API.PrintTrueFont(2 * dotsPer1mm, 5 * dotsPer1mm, "Arial", 14, 0, True, True, False, "Sample Label-1", False)

        'Draw Lines
        PvPrnWin32API.PrintBlock(1 * dotsPer1mm, 10 * dotsPer1mm, 71 * dotsPer1mm, 11 * dotsPer1mm, SLCS_BLOCK_OPTION.LINE_OVER_WRITING, 0)

        'Print string using Vector Font
        '  P1 : Horizontal position (X) [dot]
        '  P2 : Vertical position (Y) [dot]
        '  P3 : Font selection
        '        U: ASCII (1Byte code)
        '        K: KS5601 (2Byte code)
        '        B: BIG5 (2Byte code)
        '        G: GB2312 (2Byte code)
        '        J: Shift-JIS (2Byte code)
        ' P4  : Font width (W)[dot]
        ' P5  : Font height (H)[dot]
        ' P6  : Right-side character spacing [dot], Plus (+)/Minus (-) option can be used. Ex) 5, +3, -10	
        ' P7  : Bold
        ' P8  : Reverse printing
        ' P9  : Text style  (N : Normal, I : Italic)
        ' P10 : Rotation (0 ~ 3)
        ' P11 : Text Alignment
        '        L: Left
        '        R: Right
        '        C: Center
        ' P12 : Text string write direction (0 : left to right, 1 : right to left)
        ' P13 : data to print
        'PvPrnWin32API.PrintVectorFont(22, 65, "U", 34, 34, "0", False, False, False, SLCS_ROTATION.ROTATE_0, LEFTALIGN, LEFTTORIGHT, "Sample Label-2")

        PvPrnWin32API.PrintDeviceFont(2 * dotsPer1mm, 12 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_24X38, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "SHIP TO:")
        PvPrnWin32API.PrintDeviceFont(2 * dotsPer1mm, 17 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_19X30, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, True, "SATO")
        PvPrnWin32API.PrintDeviceFont(2 * dotsPer1mm, 21 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_16X25, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "7th FL, MiraeAsset Venture Tower,")
        PvPrnWin32API.PrintDeviceFont(2 * dotsPer1mm, 24 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_16X25, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "685, Sampyeong-dong, Bundang-gu,")
        PvPrnWin32API.PrintDeviceFont(2 * dotsPer1mm, 27 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_16X25, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "Seongnam-si, Gyeonggi-do,")

        PvPrnWin32API.PrintDeviceFont(3 * dotsPer1mm, 36 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_12X20, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "POSTAL CODE")
        PvPrnWin32API.Print1DBarcode(3 * dotsPer1mm, 40 * dotsPer1mm, SLCS_BARCODE.CODE39, 4 * multiplier, 6 * multiplier, 48 * multiplier, SLCS_ROTATION.ROTATE_0, SLCS_HRI.HRI_NOT_PRINT, "1234567890")

        PvPrnWin32API.PrintDeviceFont(3 * dotsPer1mm, 50 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_12X20, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "AWB:")
        PvPrnWin32API.Print1DBarcode(3 * dotsPer1mm, 53 * dotsPer1mm, SLCS_BARCODE.CODE93, 4 * multiplier, 8 * multiplier, 90 * multiplier, SLCS_ROTATION.ROTATE_0, SLCS_HRI.HRI_NOT_PRINT, "8741493121")

        PvPrnWin32API.PrintDeviceFont(4 * dotsPer1mm, 69 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_12X20, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "WEIGHT:")
        PvPrnWin32API.PrintDeviceFont(29 * dotsPer1mm, 69 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_12X20, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "DELIVERY NO:")
        PvPrnWin32API.PrintDeviceFont(53 * dotsPer1mm, 69 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_12X20, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, False, "DESTINATION")

        PvPrnWin32API.PrintDeviceFont(3 * dotsPer1mm, 73 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_32X50, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, True, "30Kg")
        PvPrnWin32API.PrintDeviceFont(26 * dotsPer1mm, 73 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_32X50, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, True, "425518")
        PvPrnWin32API.PrintDeviceFont(55 * dotsPer1mm, 73 * dotsPer1mm, SLCS_DEVICE_FONT.ENG_32X50, multiplier, multiplier, SLCS_ROTATION.ROTATE_0, True, "ICN")

        ' Prints a DATAMATRIX
        Dim xString As Integer = (35 * dotsPer1mm)
        Dim yString As Integer = (83 * dotsPer1mm)
        Dim DataMatrix_data As String = "Label Printer, This is for test."
        PvPrnWin32API.PrintDataMatrix(xString, yString, SLCS_DATAMATRIX_SIZE.DATAMATRIX_SIZE_4, False, SLCS_ROTATION.ROTATE_0, DataMatrix_data)

        ' Prints a QRCode
        '  P1 : Horizontal position (X) [dot]
        '  P2 : Vertical position (Y) [dot]
        '  P3 : MODEL selection (1, 2)
        '  P4 : ECC Level (1 ~ 4)
        '  P5 : Size of QRCode (1 ~ 9)
        '  P6 : Rotation (0 ~ 3)
        '  P7 : data to print
        Dim QRCode_data As String = "QRCode sample test 123"
        PvPrnWin32API.PrintQRCode(50 * dotsPer1mm, 83 * dotsPer1mm, SLCS_QRCODE_MODEL.QRMODEL_2, SLCS_QRCODE_ECC_LEVEL.QRECCLEVEL_M, SLCS_QRCODE_SIZE.QRSIZE_4, SLCS_ROTATION.ROTATE_0, QRCode_data)

        '// Prints a PDF417
        '//  P1 : Horizontal position (X) [dot]
        '//  P2 : Vertical position (Y) [dot]
        '//  P3 : Maximum Row Count : 3 ~ 90
        '//  P4 : Maximum Column Count : 1 ~ 90
        '//  P5 : Error Correction Level
        '//  P6 : Data compression method
        '//  P7 : HRI
        '//  P8 : Barcode Origin Point
        '//  P9 : Module Width : 2 ~ 9
        '//  P10 : Module Height : 4 ~ 99
        '//  P11 : Rotation (0 ~ 3)
        '//  P12 : data to print
        xString = (2 * dotsPer1mm)
        yString = (97 * dotsPer1mm)
        Dim PDF417_data As String = "Label Printer, This is for test."
        PvPrnWin32API.PrintPDF417(xString, yString, 8, 8, 0, 0, False, 1, 3 * multiplier, 14 * multiplier, SLCS_ROTATION.ROTATE_0, PDF417_data)

        ' Draw BOX (Fill color is black)
        PvPrnWin32API.PrintBlock(1 * dotsPer1mm, 80 * dotsPer1mm, 71 * dotsPer1mm, 112 * dotsPer1mm, SLCS_BLOCK_OPTION.BOX, 4)
        'PvPrnWin32API.PrintCircle(10, 1055, 3, 2)

        ' Print Image
        PvPrnWin32API.PrintBitmapFile(1 * dotsPer1mm, 112 * dotsPer1mm, "SATO.bmp", SLCS_DITHER_OPTION.DITHER_1, False)

        '	Print Command
        bRetVal = PvPrnWin32API.Prints(1, 1)

        Return bRetVal

    End Function



    ' byte[] -> String 
    Private Function ByteToString(ByVal strByte As Byte()) As String
        Dim str As String = Encoding.Default.GetString(strByte)
        Return str
    End Function
    ' String -> byte[] 
    Private Function StringToByte(ByVal str As String) As Byte()
        Dim StrByte As Byte() = Encoding.Default.GetBytes(str)
        Return StrByte
    End Function

    Private Sub txtP_Width_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtP_Width.KeyPress
        pInputValidateFloat(e, sender)
    End Sub

    Private Sub txtP_Height_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtP_Height.KeyPress
        pInputValidateFloat(e, sender)
    End Sub

    Private Sub txtMargin_X_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtMargin_X.KeyPress
        pInputValidateFloat(e, sender)
    End Sub

    Private Sub txtMargin_Y_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtMargin_Y.KeyPress
        pInputValidateFloat(e, sender)
    End Sub


    Private Sub txtNet_PortNum_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtNet_PortNum.KeyPress
        pInputValidateDigit(e)
    End Sub


    Private Sub rdoIF_Serial_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rdoIF_Serial.CheckedChanged
        EnableIFControls(ISerial)
    End Sub

    Private Sub rdoIF_Parallel_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rdoIF_Parallel.CheckedChanged
        EnableIFControls(IParallel)
    End Sub

    Private Sub rdoIF_USB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rdoIF_USB.CheckedChanged
        EnableIFControls(IUsb)
    End Sub

    Private Sub rdoIF_Net_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rdoIF_Net.CheckedChanged
        EnableIFControls(ILan)
    End Sub

    Private Sub rdoIF_Bluetooth_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rdoIF_Bluetooth.CheckedChanged
        EnableIFControls(IBluetooth)
    End Sub
End Class
